--[[
************************************************************************
AddOn:					Nehjo Machette
Filename:				nm.lua
Project Website:	http://www.wowace.com/addons/nehjo-machette

Copyright © 2013 Dallen Wilson <dallen.wilson@gmail.com>

nm.lua is licensed under the GNU General Public License v3.0.
The text of the GPL-3.0 license can be found in the file COPYING

This AddOn makes use of the Ace3 AddOn Development Framework.
Ace3 is licensed under the Ace3 Style BSD license.
The text of that license can be found in the file COPYING.ACE3

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
************************************************************************
]]

local _G = getfenv(0)

local NehjoMachette = _G.LibStub("AceAddon-3.0"):NewAddon("NehjoMachette", "AceConsole-3.0", "AceEvent-3.0")

local listFriends = {};
local listAcceptCustom = {};
local listInviteCustom = {};

local NehjoMachette_Version = GetAddOnMetadata("NehjoMachette", "Version");

local defaults = {
	profile = {
		master = true;
		debug = false;
		password = 'machette';
		whisper = true;
		raid = false;
		accept = true;
		acceptGuild = true;
		acceptFriends = true;
		acceptCustom = false;
		acceptCustomList = "Names,Separated,By,Commas";
		invite = false;
		inviteCustomList = "Names,Separated,By,Commas";
	}
}

local options


local function NehjoMachette_DebugPrint(msg)
	if (NehjoMachette.db.profile.debug) then print("nm:", msg); end
		
end


function NehjoMachette:GetOptions()
	if not options then
		options = {
			name = "Nehjo Machette",
			type = 'group',
			args = {
				master = {
					name = "Enable/Disable Nehjo Machette",
					desc = "Toggles the entire AddOn on or off.",
					descStyle = "inline",
					type = "toggle",
					width = "full",
					set = function(info, val) self.db.profile.master = val; if (val) then NehjoMachette:OnEnable() else NehjoMachette:OnDisable() end end,
					get = function(info) return self.db.profile.master end,
					order = 10
				},
				whisper = {
					name = "Whisper Mode",
					desc = "Toggles auto-inviting anyone who whispers the correct password.",
					descStyle = "inline",
					type = "toggle",
					width = "normal",
					set = function(info, val) self.db.profile.whisper = val end,
					get = function(info) return self.db.profile.whisper end,
					order = 20
				},
				raid = {
					name = "Whisper Raid Mode",
					desc = "Allows Whisper Mode to function in Raids.",
					type = "toggle",
					width = "normal",
					set = function(info, val) self.db.profile.password = val end,
					get = function(info) return self.db.profile.password end,
					order = 21
				},
				password = {
					name = "Whisper Password",
					desc = "Sets the password used with Whisper mode.",
					type = "input",
					width = "full",
					set = function(info, val) self.db.profile.password = val end,
					get = function(info) return self.db.profile.password end,
					order = 22
				},
				acceptCustom = {
					name = "Auto-Accept from Custom List",
					desc = "Toggles auto-accepting invites from characters on the list below.",
					descStyle = "inline",
					type = "toggle",
					width = "normal",
					set = function(info, val) self.db.profile.acceptCustom = val end,
					get = function(info) return self.db.profile.acceptCustom end,
					order = 31
				},
				accept = {
					name = "Auto-Accept Mode",
					desc = "Toggles auto-accepting incoming group invites.",
					descStyle = "inline",
					type = "toggle",
					width = "double",
					set = function(info, val) self.db.profile.accept = val end,
					get = function(info) return self.db.profile.accept end,
					order = 30
				},
				acceptCustomList = {
					name = "Auto-Accept Custom List",
					desc = "Enter a comma-separated list of names, with no spaces.",
					type = "input",
					width = "full",
					set = function(info, val) self.db.profile.acceptCustomList = val end,
					get = function(info) return self.db.profile.acceptCustomList end,
					order = 32
				},
				acceptGuild = {
					name = "Auto-Accept from Guild members",
					desc = "Toggles auto-accepting invites from Guild members.",
					descStyle = "inline",
					type = "toggle",
					width = "double",
					set = function(info, val) self.db.profile.acceptGuild = val end,
					get = function(info) return self.db.profile.acceptGuild end,
					order = 33
				},
				acceptFriends = {
					name = "Auto-Accept from Friends",
					desc = "Toggles auto-accepting invites from characters on your Friends list.",
					descStyle = "inline",
					type = "toggle",
					width = "double",
					set = function(info, val) self.db.profile.acceptFriends = val end,
					get = function(info) return self.db.profile.acceptFriends end,
					order = 35
				},	
				debug = {
					name = "Debug Mode",
					desc = "Toggles showing a -lot- of extra information in the default chat window.",
					descStyle = "inline",
					type = "toggle",
					width = "normal",
					set = function(info, val) self.db.profile.debug = val end,
					get = function(info) return self.db.profile.debug end,
					order = 36
				},
				invite = {
					name = "Auto-Invite Mode",
					desc = "Toggles auto-inviting characters from the list below. They must be RealID friends, and this will trigger when they log in.",
					descStyle = "inline",					
					type = "toggle",
					width = "double",
					set = function(info, val) self.db.profile.invite = val end,
					get = function(info) return self.db.profile.invite end,
					order = 40
				},
				inviteCustomList = {
					name = "Auto-Invite Custom List",
					desc = "Enter a comma-separated list of names, with no spaces. These characters must belong to a RealID friend.",
					type = "input",
					width = "full",
					set = function(info, val) self.db.profile.inviteCustomList = val end,
					get = function(info) return self.db.profile.inviteCustomList end,
					order = 41
				}
			}
		}
	end
        
	return options
	
end


local function NehjoMachette_OnOff(boolToggle)
	if boolToggle then
		return "Enabled";
			
	else
		return "Disabled";
			
	end
end


function NehjoMachette:ChatCommand(input)
	if not input or input:trim() == "" then
		print("|cffffff00Nehjo Machette|r v"..NehjoMachette_Version..", an addon by |cff0070deRalanar-Proudmoore|r\n");
		print("/nm |cFF33FF99master|r to enable or disable the entire addon. [|cffffff00"..NehjoMachette_OnOff(self.db.profile.master).."|r]");
		print("/nm |cFF33FF99config|r to open the gui options menu.");
		print("/nm |cFF33FF99whisper|r to toggle whisper mode [|cffffff00"..NehjoMachette_OnOff(self.db.profile.whisper).."|r]");
		print("/nm |cFF33FF99password <password>|r to set the whisper password. [|cffffff00"..self.db.profile.password.."|r]");
		print("/nm |cFF33FF99raid|r to toggle raid mode [|cffffff00"..NehjoMachette_OnOff(self.db.profile.raid).."|r]");
		print("/nm |cFF33FF99accept|r to toggle auto-accepting incoming invites. [|cffffff00"..NehjoMachette_OnOff(self.db.profile.accept).."|r]");
		print(" - /nm |cFF33FF99acceptGuild|r to toggle auto-accepting from Guild members. [|cffffff00"..NehjoMachette_OnOff(self.db.profile.acceptGuild).."|r]");
		print(" - /nm |cFF33FF99acceptFriends|r to toggle auto-accepting from Friends. [|cffffff00"..NehjoMachette_OnOff(self.db.profile.acceptFriends).."|r]");
		print(" - /nm |cFF33FF99acceptCustom|r to toggle auto-accepting from a custom list. [|cffffff00"..NehjoMachette_OnOff(self.db.profile.acceptCustom).."|r]");
		print("/nm |cFF33FF99invite|r to toggle auto-inviting RealID friends from a custom list. [|cffffff00"..NehjoMachette_OnOff(self.db.profile.invite).."|r]");
		print("/nm |cFF33FF99framehide|r to toggle automatically hiding the slide-out target marker frame. [|cffffff00"..NehjoMachette_OnOff(self.db.profile.framehide).."|r]");
		print("/nm |cFF33FF99debug|r to toggle a -lot- of extra output to the default chat window. [|cffffff00"..NehjoMachette_OnOff(self.db.profile.debug).."|r]");
	
	elseif input == "config" then
		_G.InterfaceOptionsFrame_OpenToCategory(self.optionsFrame);
		
	elseif input == "master" then
		_G.LibStub("AceConfigCmd-3.0").HandleCommand(NehjoMachette, "nm", "NehjoMachette", input)
		print("NehjoMachette: Master switch "..NehjoMachette_OnOff(self.db.profile.master));

	elseif input == "whisper" then
		_G.LibStub("AceConfigCmd-3.0").HandleCommand(NehjoMachette, "nm", "NehjoMachette", input)
		print("NehjoMachette: Whisper Mode "..NehjoMachette_OnOff(self.db.profile.whisper));
		
	elseif input == "raid" then
		_G.LibStub("AceConfigCmd-3.0").HandleCommand(NehjoMachette, "nm", "NehjoMachette", input)
		print("NehjoMachette: Raid Mode "..NehjoMachette_OnOff(self.db.profile.raid));
		
	elseif input == "accept" then
		_G.LibStub("AceConfigCmd-3.0").HandleCommand(NehjoMachette, "nm", "NehjoMachette", input)
		print("NehjoMachette: Auto-Accept Mode "..NehjoMachette_OnOff(self.db.profile.accept));
		
	elseif input == "acceptGuild" then
		_G.LibStub("AceConfigCmd-3.0").HandleCommand(NehjoMachette, "nm", "NehjoMachette", input)
		print("NehjoMachette: Auto-Accept from Guild "..NehjoMachette_OnOff(self.db.profile.acceptGuild));
		
	elseif input == "acceptFriends" then
		_G.LibStub("AceConfigCmd-3.0").HandleCommand(NehjoMachette, "nm", "NehjoMachette", input)
		print("NehjoMachette: Auto-Accept from Friends "..NehjoMachette_OnOff(self.db.profile.acceptFriends));

	elseif input == "acceptCustom" then
		_G.LibStub("AceConfigCmd-3.0").HandleCommand(NehjoMachette, "nm", "NehjoMachette", input)
		print("NehjoMachette: Auto-Accept from Custom List "..NehjoMachette_OnOff(self.db.profile.acceptCustom));

	elseif input == "invite" then
		_G.LibStub("AceConfigCmd-3.0").HandleCommand(NehjoMachette, "nm", "NehjoMachette", input)
		print("NehjoMachette: Auto-Invite Mode "..NehjoMachette_OnOff(self.db.profile.invite));
		
	elseif input == "framehide" then
		_G.LibStub("AceConfigCmd-3.0").HandleCommand(NehjoMachette, "nm", "NehjoMachette", input)
		print("NehjoMachette: Auto-Hide Target Marker Frame "..NehjoMachette_OnOff(self.db.profile.framehide));

	elseif input == "debug" then
		_G.LibStub("AceConfigCmd-3.0").HandleCommand(NehjoMachette, "nm", "NehjoMachette", input)
		print("NehjoMachette: Debug Mode "..NehjoMachette_OnOff(self.db.profile.debug));
		
	elseif string.sub(input,1,8) == "password" then
		_G.LibStub("AceConfigCmd-3.0").HandleCommand(NehjoMachette, "nm", "NehjoMachette", input)
		print("NehjoMachette: Password set to: "..self.db.profile.password);

	elseif input == "forceUpdate" then
		print("NehjoMachette: Forcing update of friendlist");
		
	elseif input == "test" then
		NehjoMachette_TEST();

	else
		_G.LibStub("AceConfigCmd-3.0").HandleCommand(NehjoMachette, "nm", "NehjoMachette", input)
		
	end
end


function NehjoMachette:OnInitialize()
	-- Load the settings
	self.db = _G.LibStub("AceDB-3.0"):New("NehjoMachetteDB", defaults, "Default")

	-- Register the options table
	_G.LibStub("AceConfig-3.0"):RegisterOptionsTable("NehjoMachette", self:GetOptions())
	self.optionsFrame = _G.LibStub("AceConfigDialog-3.0"):AddToBlizOptions("NehjoMachette", "Nehjo Machette")

	self:RegisterChatCommand("nm", "ChatCommand")
	self:RegisterChatCommand("nehjomachette", "ChatCommand")

	-- Register to receive events
	self:RegisterEvent("CHAT_MSG_WHISPER");
	self:RegisterEvent("PARTY_INVITE_REQUEST");
	self:RegisterEvent("GROUP_ROSTER_UPDATE");
	self:RegisterEvent("CHAT_MSG_BN_WHISPER");
	self:RegisterEvent("FRIENDLIST_UPDATE");
	self:RegisterEvent("BN_FRIEND_ACCOUNT_ONLINE");

end


function NehjoMachette:OnEnable()
	-- Register to receive events
	self:RegisterEvent("CHAT_MSG_WHISPER");
	self:RegisterEvent("PARTY_INVITE_REQUEST");
	self:RegisterEvent("GROUP_ROSTER_UPDATE");
	self:RegisterEvent("CHAT_MSG_BN_WHISPER");
	self:RegisterEvent("FRIENDLIST_UPDATE");
	self:RegisterEvent("BN_FRIEND_ACCOUNT_ONLINE");

end


function NehjoMachette:OnDisable()
	-- Unregister events
	self:UnregisterEvent("CHAT_MSG_WHISPER");
	self:UnregisterEvent("PARTY_INVITE_REQUEST");
	self:UnregisterEvent("GROUP_ROSTER_UPDATE");
	self:UnregisterEvent("CHAT_MSG_BN_WHISPER");
	self:UnregisterEvent("FRIENDLIST_UPDATE");
	self:UnregisterEvent("BN_FRIEND_ACCOUNT_ONLINE");

end


local function NehjoMachette_GroupCheck(event, toonName)
	if (event == nil) then event = ""; end
	if (toonName == nil) then toonName = ""; end
	
	if (UnitInParty(toonName) == 1) then 
		NehjoMachette_DebugPrint(toonName.." is already in the group");
		return false;

	elseif ((UnitIsGroupLeader("player") == false) and IsInGroup()) then
		NehjoMachette_DebugPrint("not group leader");
		
		strGroupType = nil;
			
		if (IsInRaid()) then
			strGroupType = "raid";
			
		else
			strGroupType = "party";
				
		end
			
		if ((event == "CHAT_MSG_WHISPER") or (event == "CHAT_MSG_BN_WHISPER")) then		
			partyLeader = nil;
					
			if UnitIsGroupLeader("party1") then partyLeader = UnitName("party1");
			elseif UnitIsGroupLeader("party2") then partyLeader = UnitName("party2");
			elseif UnitIsGroupLeader("party3") then partyLeader = UnitName("party3");
			elseif UnitIsGroupLeader("party4") then partyLeader = UnitName("party4");
			else partyLeader = "...actually, I don't know who's lead. Sorry!";
			end
					
			SendChatMessage("NehjoMachette: "..UnitName("player").." is not the "..strGroupType.." leader, go poke "..partyLeader, "WHISPER", nil, toonName);
			
		end
			
		return false;
			
	elseif (IsInRaid()) then
		NehjoMachette_DebugPrint("in raid");
			
		if ((event == "CHAT_MSG_WHISPER") or (event == "CHAT_MSG_BN_WHISPER")) then	
			if (NehjoMachette.db.profile.raid) then
				NehjoMachette_DebugPrint("raid invites enabled");
				return true;
					
			else
				SendChatMessage("NehjoMachette: Auto-inviting currently disabled for Raids. Sorry!", "WHISPER", nil, toonName);
				return false;
			
			end
		end
					
	elseif (GetNumGroupMembers() == 5) then 
		NehjoMachette_DebugPrint("party full");
			
		if ((event == "CHAT_MSG_WHISPER") or (event == "CHAT_MSG_BN_WHISPER")) then	
			if (NehjoMachette.db.profile.raid) then
				NehjoMachette_DebugPrint("converting to raid");
				ConvertToRaid();
				return true;
				
			else
				SendChatMessage("NehjoMachette: Party is full! Poke "..UnitName("player").." again and request he fix that.", "WHISPER", nil, toonName)
				return false;
					
			end
		
		elseif ((event == "BN_FRIEND_ACCOUNT_ONLINE")) then
			NehjoMachette_DebugPrint("unable to invite, party full.");
			
		end
			
		return false;
		
	else
		NehjoMachette_DebugPrint("groupcheck passed");
		return true;
							
	end
end


function NehjoMachette:CHAT_MSG_WHISPER(...)
	local event, arg1, arg2 = ...;
	
	if arg1 == nil then arg1 = ""; end
	if arg2 == nil then arg2 = ""; end
	
	NehjoMachette_DebugPrint(event);
	NehjoMachette_DebugPrint(arg1);
	NehjoMachette_DebugPrint(arg2);
	
	if (NehjoMachette.db.profile.whisper) then 
		if (arg1:lower():match(NehjoMachette.db.profile.password)) then
			NehjoMachette_DebugPrint("password accepted");
			
			if (NehjoMachette_GroupCheck(event, arg2)) then
				NehjoMachette_DebugPrint("inviting "..arg2);
				InviteUnit(arg2);
				
			else
				NehjoMachette_DebugPrint("groupcheck failed");
				
			end

		else
			NehjoMachette_DebugPrint("wrong password");
			
		end
	
	else
		NehjoMachette_DebugPrint("whisper-mode disabled");
		
	end
end


function NehjoMachette:CHAT_MSG_BN_WHISPER(...)
	local event, arg1, arg2 = ...;
	local breakout = false;
	
	if arg1 == nil then arg1 = ""; end
	if arg2 == nil then arg2 = ""; end
	
	NehjoMachette_DebugPrint("event: "..event);
	NehjoMachette_DebugPrint("arg1: "..arg1);
	NehjoMachette_DebugPrint("arg2: "..arg2);
	
	if ((NehjoMachette.db.profile.whisper) and (arg1:lower():match(NehjoMachette.db.profile.password))) then
		NehjoMachette_DebugPrint("password accepted");
				
		local presenceID = select(14, ...);
		local friendUnknown, friendtoonName, friendClient, realmName, friendUnknownNum, friendFaction, FriendRace, FriendClass = BNGetToonInfo(presenceID);

		if (friendClient == "WoW") then
			selfFactionName = UnitFactionGroup("player");

			if (friendFaction == selfFactionName) then
				toonName = friendtoonName.."-"..realmName;
				
				if (NehjoMachette_GroupCheck(event, toonName)) then
					NehjoMachette_DebugPrint("inviting "..toonName)
					BNInviteFriend(presenceID);
					
				else
					NehjoMachette_DebugPrint("groupcheck failed");
				
				end
			
			else
				-- not same faction as player!
				-- BNSendWhisper(presenceID, "NehjoMachette: I know the opposite faction when I see it! Group Invite: Denied!");
				NehjoMachette_DebugPrint("friend is wrong faction");						
						
			end
						
		else
			-- not wow!
			-- BNSendWhisper(presenceID, "NehjoMachette: Ohhhh you're a crafty one, you are. I know a non-WoW account when I see one! Group Invite: Denied!");
			NehjoMachette_DebugPrint("friend is non-wow client");
	
		end
		
	else
		NehjoMachette_DebugPrint("whisper mode disabled");
		
	end
end


function NehjoMachette_TEST()
	print("entered nm test function");
	wipe(listInviteCustom);
	listInviteCustom = { strsplit(",", NehjoMachette.db.profile.inviteCustomList) };
	
	for i=1, getn(listInviteCustom) do
		print(listInviteCustom[i]);

	end
end


function NehjoMachette:BN_FRIEND_ACCOUNT_ONLINE(...)
	local event, arg1, arg2 = ...;
	local breakout = false;
	
	if arg1 == nil then arg1 = ""; end
	if arg2 == nil then arg2 = ""; end
	
	if (NehjoMachette.db.profile.invite) then
		wipe(listInviteCustom);
		listInviteCustom = { strsplit(",", NehjoMachette.db.profile.inviteCustomList) };	
	
		local presenceID, givenName, idName, toonFalse, toonName = BNGetFriendInfoByID(arg1);
			
		NehjoMachette_DebugPrint("toonName is "..toonName);

		for i=1, getn(listInviteCustom) do
			if (listInviteCustom[i] == toonName) then
				NehjoMachette_DebugPrint("is on listInviteCustom");
				breakout = true;
				break
			
			end	
		end

		if breakout then
			if (NehjoMachette_GroupCheck(event, toonName)) then
				NehjoMachette_DebugPrint("inviting ",toonName);
				InviteUnit(toonName);
				
			else
				NehjoMachette_DebugPrint("groupcheck failed");
			
			end
					
		else
			NehjoMachette_DebugPrint("character not on list");
					
		end
		
	else
		NehjoMachette_DebugPrint("auto-invite mode disabled");
	
	end
end


function NehjoMachette:PARTY_INVITE_REQUEST(...)
	local event, arg1, arg2 = ...;
	local cancelFrames = true;
	local i = 1;
	
	if arg1 == nil then arg1 = ""; end
	if arg2 == nil then arg2 = ""; end
	
	NehjoMachette_DebugPrint(event);
	NehjoMachette_DebugPrint(arg1);
	
	NehjoMachette_DebugPrint("forcing friendlist update");
	NehjoMachette:FRIENDLIST_UPDATE(nil);
	
	if (NehjoMachette.db.profile.accept) then
		NehjoMachette_DebugPrint("party invite from "..arg1);

		if (NehjoMachette.db.profile.acceptGuild and UnitIsInMyGuild(arg1)) then
			-- Accept from Guild enabled and unit is in guild
			NehjoMachette_DebugPrint(arg1.." is guild member, accepting invite");
			AcceptGroup();
			
		elseif (NehjoMachette.db.profile.acceptFriends and listFriends[arg1]) then
			-- Accept from Friends enabled and unit is on friend list
			NehjoMachette_DebugPrint(arg1.." is friend, accepting invite");
			AcceptGroup();
			
		elseif (NehjoMachette.db.profile.acceptCustom) then
			wipe(listAcceptCustom);
			listAcceptCustom = { strsplit(",", NehjoMachette.db.profile.acceptCustomList) };	
	
			for i=1, getn(listAcceptCustom) do
				if (listAcceptCustom[i] == arg1) then
					NehjoMachette_DebugPrint(arg1.." is on custom list, accepting invite");
					AcceptGroup();
					break
				end
			end
			
		elseif (NehjoMachette.db.profile.acceptGuild or NehjoMachette.db.profile.acceptFriends or NehjoMachette.db.profile.acceptCustom) then
			NehjoMachette_DebugPrint("guild or friend enabled but "..arg1.." is not on list");
			-- Accept from Guild or Friends or Custom is enabled but unit is not in guild or on friends/custom list
			-- Do nothing, leave choice to user
			cancelFrames = false;
			
		else
			NehjoMachette_DebugPrint("no restrictions, accepting invite.");
			-- Auto-accept enabled but guild/friend/custom restrictions are not. Default to auto-accepting from anyone.
			AcceptGroup();
			
		end
			
		if (cancelFrames) then		
			for i=1, STATICPOPUP_NUMDIALOGS do
				local dlg = _G["StaticPopup"..i]
				if (dlg:IsVisible() and dlg.which == "PARTY_INVITE") then
					dlg.inviteAccepted = 1
					break
				end
			end
						
			StaticPopup_Hide("PARTY_INVITE");
			
		end

	else
		NehjoMachette_DebugPrint("auto-accept mode disabled");
		
	end
end


function NehjoMachette:GROUP_ROSTER_UPDATE()
	if (NehjoMachette.db.profile.framehide) then
		CompactRaidFrameManager:UnregisterAllEvents();
		CompactRaidFrameManager:Hide();
		CompactRaidFrameContainer:UnregisterAllEvents();
		CompactRaidFrameContainer:Hide();

	end
end


function NehjoMachette:FRIENDLIST_UPDATE(...)
	local numFriends = GetNumFriends();
	local numBnetFriends = BNGetNumFriends();
	local friendName = nil;
	local friendOnline = nil;
	local i = 1;
	
	wipe(listFriends);
	
	NehjoMachette_DebugPrint("Updating friends...");
	for i = 1, numFriends do
		friendName, _, _, _, friendOnline = GetFriendInfo(i);
   
		if friendOnline then
			listFriends[friendName] = true;
      
		end
	end
	
	NehjoMachette_DebugPrint("updating Bnet friends...");
	for i = 1, numBnetFriends do
		friendName, _, _, friendOnline = select(5, BNGetFriendInfo(i));
		
		if friendOnline then
			listFriends[friendName] = true;
		
		end
	end
	
	NehjoMachette_DebugPrint("Updating done.");
	
end